/*-- Copyright (c) 2015 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --*/ 
/*
Macro	  bdds_migrate
Purpose   Applies changes to a database schema, using the output of the
	  "Model Comparison Utility" (MCU). The MCU identifies differences between
	  two database schemas and generates DDL and other code to reconcile the 
	  differences. This macro applies those changes to the database schema. 
                           
DBMS          SAS and Oracle
Created       12/01/2014
Copyright (c) 2015 by SAS Institute Inc., Cary, NC 27513      
*/

/* -----------------------------------------------------------------------------------------*/
/* Parameters:	None                                                                        */
/*                                                                                          */
/* Information to be supplied by the user in the first section of the macro                 */
/*      (NOTE: no quotes should be used around any entry)                                   */
/*                                                                                          */
/* bdds_backend	        The database used by BDDS - must be either Oracle or SAS            */
/*                                                                                          */ 
/* bdds_sas_table_loc   Filesystem location of the table files (SAS only)                   */
/*                                                                                          */
/* bdds_uname	        The Oracle user ID that will run the schema update                  */
/*                       (supply only if authdomain not in use)                             */
/*                                                                                          */
/* bdds_upw	        The password of the Oracle user ID                                  */
/*                       (supply only if authdomain is not in use)                          */
/*                                                                                          */ 
/* bdds_ora_path	The Oracle Path associated with the database                        */
/*                                                                                          */ 
/* bdds_schema	        The Oracle schema associated with the database                      */
/*                                                                                          */ 
/* authdomain	        The authdomain that will establish credentials                      */
/*                        (do not fill in User ID or password if authdomain is used)        */
/*                                                                                          */ 
/* bdds_comp_script_path_name    The filesystem location of the schema comparison .csv      */
/*                               file supplied with this solution                           */
/*                               bdds_comp_label_script_path_name                           */
/*                               The filesystem location of the label comparison .csv file  */
/*                               supplied with this solution (SAS only)                     */                                                 /*                                                                                          */
/* bdds_libref	        The libref for the Detail Data Store - default is DDS               */
/*                                                                                          */ 	
/* fileloc		The filesystem location of the comparison scripts                   */
/*                      (e.g. ALTER scripts) supplied with this solution                    */
/*                                                                                          */ 
/*----------------------------------------------------------------------------------------- */ 

%macro bdds_migrate; 

%global sda_ddl_filepath bdds_comp_script_path_name bdds_comp_label_script_path_name bdds_schema bdds_backend_db bdds_sas_table_loc
        bdds_libref libref bdds_smdfile smdfile bdds_uname bdds_upw bdds_libpath bdds_ora_path DTTMFMT FILELOC TMFMT DTFMT FMTRK;

/* ---------------------------------------------------------------------------- */ 
/*      Update this section adding the parameters as identified below           */ 
/* ---------------------------------------------------------------------------- */ 
/* ---------------------------------------------------------------------------- */ 
/*	Backend database - MUST be either Oracle or SAS	(no quotes)		*/
/* ---------------------------------------------------------------------------- */ 

%let bdds_backend_db = ;

/* ---------------------------------------------------------------------------- */ 	
/*   Filepath of Existing Banking DDS tables (SAS only - leave blank for Oracle)*/
/*   Example: %let bdds_sas_table_loc= D:\BankingDDS_4.7\Physical_4.7_tables;   */ 
/* ---------------------------------------------------------------------------- */ 

%let bdds_sas_table_loc= ;

/* ----------------------------------------------------------------------------*/ 
/*	Oracle login/connect info: username, password, path, and schema name   */
/*	NOTE: If authdomains are in use, do NOT specify the username           */
/*      (bdds_uname) orpassword (bdds_upw). IF authdomains are not in use,     */
/*      provide all information except for the authdomain.                     */
/*---------------------------------------------------------------------------- */ 


%if %kupcase(&bdds_backend_db)=ORACLE %then %do;
	%let bdds_uname= ;
	%let bdds_upw=  ;
	%let bdds_ora_path= ;
	%let bdds_schema= ;
	%let authdomain= ; 
%end;

/*--------------------------------------------------------------------------------------------------- */ 
/* 	CSV Comparison file path and name.  Must remain a csv file (do not convert to xls or xlsx)    */ 
/*	Example: <your install location>\bankingdds_4.7\upgrade\ddl\ModelComparisonResults_7to7HF.csv */
/*--------------------------------------------------------------------------------------------------- */ 

%let bdds_comp_script_path_name = ;

/*--------------------------------------------------------------------------------------------------------- */ 
/*     CSV Comparison Label file path and name.  Must remain a csv file (do not convert to xls or xlsx)     */ 
/*     Example: <your install location>\bankingdds_4.7\upgrade\ddl\ModelComparisonResults_7to7HF_labels.csv */
/*--------------------------------------------------------------------------------------------------------- */ 

%let bdds_comp_label_script_path_name = ;

/*----------------------------------------------------------------------------------------- */ 
/*    Libref for the banking dds - change only if necessary		     		    */ 	
/*----------------------------------------------------------------------------------------- */ 

%let bdds_libref=DDS;

/*------------------------------------------------------------------------------------ */
/*   Upgrade Create/Alter DDL Location 	                                               */						
/*   Example: %let fileloc= <your install location>\bankingdds_4.7\upgrade\ddl\oracle\;*/
/*   Example: %let fileloc= <your install location>\bankingdds_4.7\upgrade\ddl\sas\;   */
/*------------------------------------------------------------------------------------ */

%let fileloc = ; 

/* --------------------------------------------- */ 
/* !!!!!!!  Do not update entries below  !!!!!!! */ 
/* --------------------------------------------- */ 


/* Datetime format */ 
%let DTTMFMT=NLDATM21.;

/* Time format */ 
%let TMFMT=NLTIMAP11.;

/* Date format */ 
%let DTFMT=DATE9.;

/* Format track */ 
%let FMTRK=12.;


/* ------------------------------------- */ 
/*  Use/modify some user-entered values	 */
/* ------------------------------------- */ 

%let schema=%str(&bdds_schema..);
%let libref=&bdds_libref;
%let bdds_backend_db = %kupcase(&bdds_backend_db); 

/* ---------------------------------------------------------------------------- */
/* Validate Parameters In This Section                                          */ 
/* ---------------------------------------------------------------------------- */

/*	Valid SAS or Oracle entry	*/

%if (&bdds_backend_db ne ORACLE and &bdds_backend_db ne SAS) %then %do;
    %put Error:  Backend database must be either oracle or sas - process stopped; 
    %goto finish;
%end;

/* ----------------------------------------------- */
/*	Checking path names                        */
/* ----------------------------------------------- */

/* ----------------------------------------------------------- */
/*	Process the existing SAS tables	                       */
/* ----------------------------------------------------------- */

%if &bdds_backend_db eq SAS %then %do;
	%dir_check(&bdds_sas_table_loc);
	%if rc=0 %then %goto finish;
%end;


/* ----------------------------------------------------------- */
/*	Verify the "changes" spreadsheets	               */
/* ----------------------------------------------------------- */

%if &bdds_backend_db eq SAS %then %do;
	%dir_check(&bdds_comp_label_script_path_name);
	%if rc=0 %then %goto finish;
%end;

%dir_check(&bdds_comp_script_path_name);
%if rc=0 %then %goto finish;

/* ----------------------------------------------------------- */
/*	Verify the locations of the comparison script files    */
/* ----------------------------------------------------------- */

%dir_check(&fileloc);
%if rc=0 %then %goto finish;


/* ----------------------------------------------------------- */
/*	 Verify the libname exists                             */
/* ----------------------------------------------------------- */

%if &bdds_backend_db=ORACLE %then %do;
	libname &bdds_libref ORACLE 
	%if &authdomain eq %str() %then %do;
		USER=&bdds_uname 
		PASSWORD=&bdds_upw 
	%end;
	%else %do;
		AUTHDOMAIN=&authdomain
	%end;
	PATH=&bdds_ora_path;
	%let rc = %sysfunc(libref(&libref));
	%if &rc %then %do;	
	   %put ERROR:  &libref Library does not exist.  Process ending.;
	   %goto finish;
	%end; 	
%end;
%else %do;
	libname &bdds_libref "&bdds_sas_table_loc";
	%let rc = %sysfunc(libref(&bdds_libref));
	%if &rc %then %do;
	   %put ERROR:  &libref Library does not exist.  Process ending.;
	   %goto finish;
	%end; 	
%end;


/* ---------------------------------------------------- */ 
/*	Check for database                              */
/* ---------------------------------------------------- */ 

%if &bdds_backend_db=ORACLE %then %do;
	proc sql noprint;
		connect to oracle (
		%if &authdomain eq %str() %then %do;
			user=&bdds_uname 
			password=&bdds_upw 
		%end;
		%else %do;
			AUTHDOMAIN=&authdomain
		%end;
		path=&bdds_ora_path);	
	%if &rc %then %do;
	   %put ERROR:  Database does not exist.  Process ending.;
	   %goto finish;
	%end; 	
%end;	

/*	For SAS, look for a table that MUST be there - if it's not, assume the database
	does not exist																	*/

%else %do;
	%if %sysfunc(exist(&bdds_libref..COUNTERPARTY))=0 %then %do;   
	   %put ERROR:  Database does not exist.  Process ending.;
	   %goto finish;
	%end; 	
%end;

/* ------------------------------------------------- */ 
/*	End of validation                            */                   
/* ------------------------------------------------- */ 


/* ------------------------------------------------- */ 
/* 	Begin Functional scripts                     */  
/* ------------------------------------------------- */ 

/* ---------------------------------------------------------------------------- */ 
/*     Import Change Files sub-section                                          */  
/*                                                                              */
/* 	Import the Comparison Tool output into the created tables.              */
/*	NOTE 1:                                                                 */
/*        Source file must be saved as .csv to allow import into systems.       */
/*	regardless of the host OS (e.g. Unix will not read a Windows Excel file)*/
/*	NOTE 2:                                                                 */
/*        Each tab of the source file spreadsheet must be saved separately.     */
/*	Neither the RANGE nor the SHEET parameter can be used in the import     */
/*	to distinguish different sheets of the original workbook.               */
/*	The utillity imports the schema changes, then the label changes.        */
/*	NOTE 3:                                                                 */
/*      The FILENAME preceding the PROC IMPORT is necessary to make sure that   */
/*	the host importing the file is aware that lines are terminated with a   */
/*	LF. This should allow the file to be successfully imported              */
/*	into a Unix host as well as a Windows host.		                */
/* ---------------------------------------------------------------------------- */ 

FILENAME infile "&bdds_comp_script_path_name" termstr=LF;
PROC IMPORT 
	DATAFILE=infile 
	OUT=work.schema_change
	DBMS=CSV replace;
	GETNAMES=no;
	GUESSINGROWS=200;   /* A large number */
	DATAROW=4;
RUN;

/* ------------------------------------------------------- */ 
/* Return Code Check - if Error then do not go any further */ 
/* ------------------------------------------------------- */ 

%bdds_migrate_checkerr; 
%if %eval(&bdds_migrate_tmprc) > 4 %then %do; 
    %put Error:  Error found proc import of comparison script - process stopped; 
    %goto finish;
%end;

/* ------------------------------------------------------- */ 
/*    Process the label changes if necessary               */
/* ------------------------------------------------------- */ 

%if &bdds_backend_db=SAS %then %do;
	FILENAME infile1 "&bdds_comp_label_script_path_name" termstr=LF;
	PROC IMPORT 
		DATAFILE=infile1
		OUT=work.label_change
		DBMS=CSV replace;
		GETNAMES=no;
		GUESSINGROWS=200;   /* Large Nmber  */
		DATAROW=3;
	RUN;
	%bdds_migrate_checkerr; 
	%if %eval(&bdds_migrate_tmprc) > 4 %then %do; 
	    %put Error:  Error found proc import of label script - process stopped; 
	    %goto finish;
	%end;
%end;


/* --------------------------------------------------------------------------------------------- */ 
/*get lengths for each column to make sure enough buffer space is allocated to process the table */
/* --------------------------------------------------------------------------------------------- */ 

DATA work.schema_change;
	SET work.schema_change;
	tab_len=length(var1);
	act_len=length(var2);
	obj_len=length(var3);
run;
%if &bdds_backend_db=SAS %then %do;
	DATA work.label_change;
		SET work.label_change;
		var_len=length(var1);
		act_len=length(var2);
		tbl_len=length(var3);
	run;
%end;

/* ---------------------------------------------------------------------------- */ 
/*     Create New Tables                                                        */  
/* ---------------------------------------------------------------------------- */

/*	First we'll sort by the type of change and the column length. The
	idea is to have the longest one come first */	

	proc sort data=work.schema_change out=work.schema_change;
	   by VAR2 tab_len;	
	run;

/*  Get the number of new tables to be (potentially) created  */

	%global NewTableCount;
	proc SQL NOPRINT;
		SELECT COUNT(*) into :NewTableCount
			FROM work.schema_change
			WHERE VAR2 = "ADDITIONAL TABLE";
		SELECT VAR1 into :NewTableName1 -
			FROM work.schema_change
			WHERE VAR2 = "ADDITIONAL TABLE";
	quit;
	DATA _NULL_ ;
	%do i=1 %to &NewTableCount;
		%let dsname=&&NewTableName&i;
		%bdds_migrate_opends(&dsname);
	%end;
	
/*	Sort by the table name, the change and the column length.  */	

	
	proc sort data=work.schema_change out=work.schema_change;
	   by VAR1 VAR2 descending obj_len;	
	run;
	

	%Global ColFound q;
	%local CheckFoundColumn;
	%let ColFound=0;
    proc sql noprint;

/*  Determine how many tables have new columns   */

		select count (distinct (VAR1))
		    into :NewColNumTbl
		    from work.schema_change
		    where var2 = 'ADDITIONAL COLUMN';


/*  Get the number of new columns per table   */

	    select var1, count(*)
	        into :tabname1- , :newcolnum1-
	        from work.schema_change
	        where var2 = 'ADDITIONAL COLUMN'
	        group by var1;
               
/*  Select the table and column names into macro variables  */

	    select VAR1, VAR3
	        into :tabcolname1- ,  :colname1-
	        from work.schema_change
	        where VAR2 = 'ADDITIONAL COLUMN'
	        order by VAR1, VAR3;
		quit;

/*	Check if the tables that hold already-existing columns have been created yet and
	either create them or empty them if they exist	*/

		%if %sysfunc(exist(work.ColumnExistsTmp))=0 %then %do;   
			proc sql;
			create table work.ColumnExistsTmp (
				ExistTable	char(32),
				ExistColumn	char(32)
			);
			quit;
		%end;
		%else %do;		
			proc sql;
				delete
				from work.ColumnExistsTmp;
			quit;
		%end;

		/* Return Code Check - if Error then do not go any further */ 

		%bdds_migrate_checkerr; 
		%if %eval(&bdds_migrate_tmprc) > 4 %then %do; 
		    %put Error:  Error found creating ColumnExists table - process stopped; 
		    %goto finish;
		%end;


		%if %sysfunc(exist(work.ColumnExistsFull))=0 %then %do;   
			proc sql;
			create table work.ColumnExistsFull (
				ExistTable	char(32),
				ExistColumn	char(32)
			);
			quit;
		%end;
		%else %do;		
			proc sql;
				delete
				from work.ColumnExistsFull;
			quit;
		%end;

/* ------------------------------------------------------- */ 
/* Return Code Check - if Error then do not go any further */ 
/* ------------------------------------------------------- */ 

		%bdds_migrate_checkerr; 
		%if %eval(&bdds_migrate_tmprc) > 4 %then %do; 
		    %put Error:  Error found creating ColumnExists table - process stopped; 
		    %goto finish;
		%end;

/* --------------------------------------------------------- */ 
/* execute the test to see if the column exists in the table */
/* --------------------------------------------------------- */ 

	%let LoopCount=1;
    %do i=1 %to &NewColNumTbl;		                                /* Once for each table with potential new columns */
		%let CheckFoundColumn=0;	
        %do q=1 %to %eval(&&newcolnum&i); 	                        /* Once for each new column in the table */
			%let CheckFoundColumn=&ColFound;		/* Before checking for the column */
        	%bdds_migrate_varexist(&&tabname&i, &&colname&LoopCount);
			%if %eval(&CheckFoundColumn<&Colfound) %then %do;              /*  column found*/
				%put Table &&tabname&i column &&colname&LoopCount exists!;
				%bdds_migrate_addfoundcol(&&tabname&i, &&colname&LoopCount);
			%end;
			%let LoopCount=%eval(&LoopCount+1);
		%end;
		%if &ColFound=0 %then %do;	
			%bdds_migrate_applyalter(&&tabname&i);	
		%end;		
		%else %do; 

/* -------------------------------------------------------------------------------- */ 
/*	Remove the columns that already exist from the set of ALTER TABLE scripts   */		
/*	and then execute any remaining ALTER TABLE commands                         */
/* -------------------------------------------------------------------------------- */ 

	           	proc sort data=work.ColumnExistsTmp out=work.ColumnExistsTmp;
		   		by ExistTable ExistColumn;
		   	run;	
		   
		   /*	Column Count	*/
		   
		   	proc sql noprint;
		   		select count(*)
		   		into :ExistColumnCount	
		   		from work.ColumnExistsTmp;
		   
		   /*	Add the table/column names into a variable array	*/
		   
		   		select ExistTable, ExistColumn
		   		into :ExistTab1- , :ExistCol1-	
		   		from work.ColumnExistsTmp;
		   	quit;
		   
		   /*	feed these table and column names to the proc that will delete
		   	references to them from the ALTER TABLE scripts	*/
		   
		   	%do k=1 %to &ExistColumnCount;
		   		%bdds_migrate_RemoveExistColRefs(&&ExistTab&k, &&ExistCol&k);		
	                %end;
	                
			%bdds_migrate_exec_remaining_cols(&&tabname&i);
			%let ColFound=0;		                     /* Reset the counter */

/* Copy the contents into the Found table and empty the working table */

			proc sql;
				insert into work.ColumnExistsFull 
					select * from work.columnexiststmp;
				delete from work.columnexiststmp;
			quit;
		%end;				
    %end;	

/* --------------------------------------------------------------------- */ 
/*    Label Change Section - SAS Specific                                */ 
/* --------------------------------------------------------------------- */ 
/*	_label_change actually performs the update to the                */
/*	label value using proc datasets. It needs to know                */
/*	the table, the column and the new label value	                 */		
/*                                                                       */
/*	call_label_parse determines how many label changes there         */
/*	are to process, then collects the table, label and column        */
/*	information into variable to be used by the other procs.         */
/*	It then calls the other two procs to parse and change the        */
/*	labels.	                                                         */
/* --------------------------------------------------------------------- */ 							
	
    %global label_count;
	%if &bdds_backend_db=ORACLE %then %do;
		%goto finish;
	%end;
		
    proc sql noprint;
		select count(*)
		into :label_count
		from work.label_change;
		select VAR1, VAR2, VAR3
		into :tabname1- , :full_label1- , :colname1-
		from work.label_change;
    quit;
    %do i=1 %to &label_count;
	%bdds_migrate_label_parse(label_string=&&full_label&i);
	%bdds_migrate_label_change(chgtable=&&tabname&i, chgcol=&&colname&i, newlabel=&to_label);
    %end;    
	
%finish:  %put %str();

%mend ;  /*bdds_migrate  */

/* ------------------------------------ */ 
/* Supporting Macros used in above code */ 
/* ------------------------------------ */ 

%macro bdds_migrate_opends(tabname);
%local table_exists table_string;
%if &bdds_backend_db=ORACLE %then %do;
	%let table_string=%sysfunc(cats(&bdds_libref, ., &tabname));
	%end;
%else %do;
	%let table_string=%sysfunc(cats(&bdds_libref, ., &tabname));
	%end;
%let table_exists=%sysfunc(exist(&table_string));	/* 1 -> exists, >0 -> does not exist  */
%if &table_exists ne 1 %then %do;
/* check that the slash is included in the filepath name.  If not add it */ 
	%let pathdel =; 
	data _null_;
		length text $3000;
		text = left(trim(reverse("&fileloc")));
		endvar = ksubstr(text,1,1);
		if endvar ne '/' and endvar ne '\' then do;
			call symput('PATHDEL','/');
		end;
	run; 
    %if "&pathdel" ne "" %then %do; 
	    %let fileloc_alt = &fileloc&pathdel; 
    %end;
    %else %do; 
    	%let fileloc_alt = &fileloc; 
    %end; 
	%let lc_dsname=%klowcase(&dsname);
	%let include_string=%sysfunc(quote(%sysfunc(cats(&fileloc, &lc_dsname, .sas)))); 
	PROC SQL NOPRINT;
		%if &bdds_backend_db=ORACLE %then %do;
			connect to oracle (
			%if &authdomain eq %str() %then %do;
				user=&bdds_uname 
				password=&bdds_upw 
			%end;
			%else %do;
				AUTHDOMAIN=&authdomain
			%end;
			path=&bdds_ora_path);
		%end;
		%include &include_string;
	QUIT;
%end;
%else %put Table &table_string already exists;
%mend  bdds_migrate_opends; 

/* ------------------------------------ */ 
/*   Variable existence check           */ 
/* ------------------------------------ */ 
%macro bdds_migrate_varexist(ds,var);
        %global lib dsid ColFound;
		%let lib=dds;
        %let lib_table = %kupcase(&lib..&ds);    
        %let dsid=%sysfunc(open(&lib_table));  
		%if &dsid=0 %then %do;
			%let tabnote=Table &lib_table not found!;
			%put &tabnote;
			%return;
		%end;

/* -------------------------------------- */ 
/*	test to see if the column exists. */
/* -------------------------------------- */ 

        %if %sysfunc(varnum(&dsid,&var)) > 0 %then %let ColFound=%eval(&ColFound+1);	/*  Table and column exists  */
        %let rc=%sysfunc(close(&dsid));
%mend bdds_migrate_varexist; 


%Macro bdds_migrate_addfoundcol(TabToAdd, ColToAdd);

/* --------------------------------------------------------------------------- */ 
/*	Add the table/column names of existing columns to the working table    */
/* --------------------------------------------------------------------------- */ 

	proc sql noprint;
		insert into work.ColumnExistsTmp (ExistTable, ExistColumn)
		values ("&TabToAdd", "&ColToAdd");
	quit;
%mend bdds_migrate_addfoundcol;

%macro bdds_migrate_applyalter(AlterTable);

/* --------------------------------------------------------------------------- */ 
/* For those tables that have no existing columns, run the ALTER scripts as-is */	
/* Set up all the necessary variables prior to running them.                   */
/* --------------------------------------------------------------------------- */ 
	%local ApplyFile lc_AlterTable;
	%let lc_AlterTable=%klowcase(&AlterTable);
	%let ApplyFile=%sysfunc(cats(alter_, &lc_AlterTable, .sas));
	proc sql;
		%if &bdds_backend_db=ORACLE %then %do;
			connect to oracle (
			%if &authdomain eq %str() %then %do;
				user=&bdds_uname 
				password=&bdds_upw 
			%end;
			%else %do;
				AUTHDOMAIN=&authdomain
			%end;
			path=&bdds_ora_path);		
		%end;
		%include "&fileloc&ApplyFile";
	quit;
%mend;

%macro bdds_migrate_removeexistcolrefs(tablename , columnname );

/* ------------------------------------------------------------ */ 
/*	Construct the path and filename for the ALTER file	*/
/* ------------------------------------------------------------ */ 

	%local alterfile infilespec ;
	%let lc_tablename=%klowcase(&tablename);
	%let alterfile=%sysfunc(cats(alter_, &lc_tablename, .sas));
	%let infilespec=%sysfunc(cats(&fileloc, &alterfile));		

/* ------------------------------------------------------------------------------------	*/	
/*	Read in the file with all the ALTER TABLE statements and store                  */
/*	in a SAS dataset. Note the filter so that any statement (comment, index, etc.)  */
/*	that doesn't have ALTER TABLE in it will not be written to the dataset	        */
/* ------------------------------------------------------------------------------------	*/	

	%if &k=1 %then %do;
		DATA work.&tablename ;
			INFILE "&infilespec" length=reclen;
			INPUT SQLLINE $VARYING200.reclen;
			IF INDEX(SQLLINE, "ALTER TABLE")>0;
		RUN;
/* ------------------------------------------------------------------------------------	*/
/*	Use the created dataset to search for the existing column name. If it's found,  */
/*	delete the line in the file that contains the column name			*/
/* ------------------------------------------------------------------------------------	*/

		DATA work.&tablename;
			SET work.&tablename;
			IF INDEX(SQLLINE, "&columnname") >0 THEN DELETE;
		RUN;
	%end;

/* ------------------------------------------------------------------------------------	*/
/*	Repeat the search and delete steps for the remaining existing column names	*/
/* ------------------------------------------------------------------------------------	*/

	%else %do;
		DATA work.&tablename;
			SET work.&tablename;
			IF INDEX(SQLLINE, "&columnname") >0 THEN DELETE;
		RUN;
	%end;
%mend bdds_migrate_removeexistcolrefs;

%macro bdds_migrate_Exec_remaining_cols(ColumnSet);
	proc sql noprint;
		select SQLLINE
		into :AddColLine1-
		from work.&ColumnSet
		where substr(SQLLINE,1,2) NE "/*";  /*	Exclude any comment lines	*/
	quit;
	%if &sqlobs > 0 %then %do;
		%let ColsToProcess=&sqlobs;
		%do s=1 %to &ColsToProcess;
			proc sql noprint;
				%if &bdds_backend_db=ORACLE %then %do;
					connect to oracle (
					%if &authdomain eq %str() %then %do;
						user=&bdds_uname 
						password=&bdds_upw 
					%end;
					%else %do;
						AUTHDOMAIN=&authdomain
					%end;
					path=&bdds_ora_path);				
				%end;
				&&AddColLine&s
			quit;
		%end;
	%end;
%mend bdds_migrate_exec_remaining_cols;

%macro bdds_migrate_label_change(chgtable=, chgcol=, newlabel=);
proc datasets lib=dds nolist;
    modify &chgtable;
    label &chgcol=%sysfunc(quote(&newlabel));
	run; 
quit;
%mend bdds_migrate_label_change;

%macro bdds_migrate_label_parse(label_string=);
/*	_label_parse teases out the value of the new value for
	the label that is embedded in a string output from the
	model comparison utility. The string is <old label value>
	TO <new label value> so we have to find that "TO" as 
	the starting point for parsing the new value. */
	
	%local label_start full_lab_len to_lab_len;
	%global to_label;
	%let label_start=%sysfunc(INDEX(&label_string, TO));
	%let label_start=%eval(&label_start+3); /* move past TO	*/
	%let full_lab_len=%sysfunc(length(&label_string));
	%let to_lab_len=%eval(&full_lab_len - &label_start + 1);
	%let to_label=%sysfunc(substr(&label_string, &label_start, &to_lab_len));
%mend bdds_migrate_label_parse;	

/*	Directory checking macro	*/

%macro dir_check(pathname);
%if %sysfunc(length(&pathname)) = 0 %then %do;
		%put ERROR: An entry is required for &pathname. Process ending.;
		%let rc=0;
%end; 	
%else %let rc = %sysfunc(filename(fileref,&pathname)) ; 
%if %sysfunc(fexist(&fileref))=0 %then %do;   
		%put ERROR: The file or filepath &pathname does not exist.  Process ending.;
		%let rc=0;
%end;
%mend;

/* Error Check macro */ 

%macro bdds_migrate_checkerr;
  %global bdds_migrate_tmprc;
  %let bdds_migrate_tmprc = 0;

  %if (&sysrc ne 0) or (&syserr ne 0 and &syserr ne 4) %then %do;
    %if (&sysrc ne 0) %then
        %let bdds_migrate_tmprc = &sysrc;
    %else
        %let bdds_migrate_tmprc = &syserr;
  %end;

%mend bdds_migrate_checkerr;
